@@ -16,6 +16,7 @@
 #include "EXTERN.h"
 #include "common.h"
 #include "util.h"
+#include "util2.h"
 #include "final.h"
 #include "help.h"
 #include "cache.h"
@@ -27,6 +28,9 @@
 #undef TIOCGWINSZ
 #endif
 
+#undef	USETITE		/* use terminal init/exit seqences (not recommended) */
+#undef	USEKSKE		/* use keypad start/end sequences */
+
 char ERASECH;		/* rubout character */
 char KILLCH;		/* line delete character */
 char tcarea[TCSIZE];	/* area for "compiled" termcap strings */
@@ -148,19 +152,23 @@ char *tcbuf;		/* temp area for "uncompiled" termcap entry */
 #if ! defined (FIONREAD) && ! defined (HAS_RDCHK)
     /* do no delay reads on something that always gets closed on exit */
 
-    devtty = open("/dev/tty",0);
-    if (devtty < 0) {
-	printf(cantopen,"/dev/tty") FLUSH;
-	finalize(1);
+    devtty = fileno(stdin);
+    if (isatty(devtty)) {
+	devtty = open("/dev/tty",0);
+	if (devtty < 0) {
+	    printf(cantopen,"/dev/tty") FLUSH;
+	    finalize(1);
+	}
+	fcntl(devtty,F_SETFL,O_NDELAY);
     }
-    fcntl(devtty,F_SETFL,O_NDELAY);
 #endif
 #endif
     
     /* get all that good termcap stuff */
 
 #ifdef HAS_TERMLIB
-    status = tgetent(tcbuf,getenv("TERM"));	/* get termcap entry */
+    s = getenv("TERM");
+    status = tgetent(tcbuf,s? s : "dumb");	/* get termcap entry */
     if (status < 1) {
 #ifdef VERBOSE
 	printf("No termcap %s found.\n", status ? "file" : "entry") FLUSH;
@@ -188,6 +196,8 @@ char *tcbuf;		/* temp area for "uncompiled" termcap entry */
     CE = Tgetstr("ce");			/* clear to end of line string */
     TI = Tgetstr("ti");			/* initialize display */
     TE = Tgetstr("te");			/* reset display */
+    KS = Tgetstr("ks");			/* enter `keypad transmit' mode */
+    KE = Tgetstr("ke");			/* exit `keypad transmit' mode */
     HO = Tgetstr("ho");			/* home cursor */
     IL = Tgetstr("al");			/* insert (add) line */
     CM = Tgetstr("cm");			/* cursor motion */
@@ -1331,6 +1341,7 @@ int dummy;
     /* Well, if SIGWINCH is defined, but TIOCGWINSZ isn't, there's    */
     /* almost certainly something wrong.  Figure it out for yourself, */
     /* because I don't know how to deal with it :-)                   */
+    ERROR!
 #endif
 }
 #endif
@@ -1344,6 +1355,12 @@ termlib_init()
 	fflush(stdout);
     }
 #endif
+#ifdef USEKSKE
+    if (KS && *KS) {
+	tputs(KS,1,putchr);
+	fflush(stdout);
+    }
+#endif
 }
 
 void
@@ -1355,6 +1372,12 @@ termlib_reset()
 	fflush(stdout);
     }
 #endif
+#ifdef USEKSKE
+    if (KE && *KE) {
+	tputs(KE,1,putchr);
+	fflush(stdout);
+    }
+#endif
 }
 
 void
